theory Formula
imports Main Interpretation
begin

type_synonym atom = bool

datatype binop = And | Or | Impl

(* inductive definition of abstract syntax trees for formula *)

datatype form = 
     Var  "var" 
   | Atom "atom"
   | Neg  "form"
   | Bin  "binop" "form" "form"


text \<open> Notations \<close>

syntax        "_top"  :: "form"   ("top\<^sub>F")
translations  "top\<^sub>F"  == "(CONST Atom) (CONST True)"

syntax        "_bot"  :: "form"   ("bot\<^sub>F")
translations  "bot\<^sub>F"  == "(CONST Atom) (CONST False)"

syntax        "_neg"  :: "form \<Rightarrow> form"   ("neg\<^sub>F _")
translations  "neg\<^sub>F F" == "(CONST Neg) F"

syntax        "_and"  :: "form \<Rightarrow> form \<Rightarrow> form"   ("and\<^sub>F _ _")
translations  "and\<^sub>F F1 F2" == "(CONST Bin) (CONST And) F1 F2"

syntax        "_or"  :: "form \<Rightarrow> form \<Rightarrow> form"   ("or\<^sub>F _ _")
translations  "or\<^sub>F F1 F2" == "(CONST Bin) (CONST Or) F1 F2"

syntax        "_impl"  :: "form \<Rightarrow> form \<Rightarrow> form"   ("impl\<^sub>F _ _")
translations  "impl\<^sub>F F1 F2" == "(CONST Bin) (CONST Impl) F1 F2"

(** Test cases *)

definition p0 where "p0 = Var 0"
definition p1 where "p1 = Var 1"
definition p2 where "p2 = Var 2"

definition P0     where "P0        = impl\<^sub>F p0 p1"
definition P1     where "P1        = impl\<^sub>F p1 P0"
definition dnegp  where "dnegp P Q = (impl\<^sub>F (impl\<^sub>F P Q) Q)"
definition P2     where "P2        = dnegp P0 p0"

(** ** Semantic *)
(* Boolean interpretation of binary operators *)

fun interp where
    "interp I (Var x) = I x"
  | "interp I (Atom a) = a"
  | "interp I (Neg Q)  = (\<not> (interp I Q))"
  | "interp I (Bin And Q R)  = ((interp I Q) \<and> (interp I R))"
  | "interp I (Bin Or Q R)   = ((interp I Q) \<or> (interp I R))"
  | "interp I (Bin Impl Q R) = ((interp I Q) \<longrightarrow> (interp I R))"


value "interp Itrue P2"
value "interp Ifalse P2"
value "interp (interpl [True,False]) P2"
value "interp (interpl [False,True]) P2"

(*
Definition eval2 A it (P : A) : list bool := 
  (it Itrue P)::(it Ifalse P)::
  (it (interpl (true::false::nil)) P)::
  (it (interpl (false::true::nil)) P)::nil.
Eval compute in (eval2 interp P2).

Definition truthtable2 A it (P : A) : list (bool * bool * bool) := 
  (true,true,it Itrue P)::(false,false,it Ifalse P)::
  (true,false,it (interpl (true::false::nil)) P)::
  (false,true,it (interpl (false::true::nil)) P)::nil.


Eval compute in (truthtable2 interp (orp (negp p0) p1)).
Eval compute in (truthtable2 interp (andp (negp p0) p1)).
Eval compute in (truthtable2 interp (implp (negp p0) p1)).
*)
(** ** Equivalence *)

definition equiv where "equiv P Q = (\<forall>I. interp I P = interp I Q)"

lemma negimpl : "equiv (neg\<^sub>F (impl\<^sub>F P Q)) (and\<^sub>F P (neg\<^sub>F Q))"
unfolding equiv_def
by auto



lemma implor : "equiv (impl\<^sub>F P Q) (or\<^sub>F (neg\<^sub>F P)  Q)" 
unfolding equiv_def
by auto


(** ** Validity *)
definition valid where "valid P = (\<forall> I. interp I P = True)"

lemma Pierce :" valid (impl\<^sub>F (impl\<^sub>F (impl\<^sub>F P Q) P) P)"
unfolding valid_def
by auto

end
